@@ -11,7 +11,7 @@ Graph::ModularDecomposition - Modular decomposition of directed graphs
 =cut
 
 require Exporter;
-our $VERSION = '0.13';
+our $VERSION = '0.14';
 
 use Graph 0.20105;
 require Graph::Directed;
@@ -72,10 +72,10 @@ or unclassified (should never apply).
 
 =head2 RELATED WORK
 
-Several recent graph algorithms have used the modular decomposition
-tree as a basic building block.  A simple example application of
-these routines is to construct and search the modular decomposition
-tree of a directed graph to determine if it is node-series-parallel.
+Several graph algorithms use the modular decomposition tree as a
+building block.  A simple example application of these routines is
+to construct and search the modular decomposition tree of a directed
+graph to determine if it is node-series-parallel.
 Checking if a digraph is series-parallel can also be determined using
 the O(m+n) Valdes-Tarjan-Lawler algorithm published in 1982, but this
 only constructs a decomposition tree if the input is series-parallel:
@@ -87,17 +87,16 @@ two-structures, from
 A. Ehrenfeucht, H. N. Gabow, R. M. McConnell, and S. J. Sullivan, "An
 O(n^2) Divide-and-Conquer Algorithm for the Prime Tree Decomposition
 of Two-Structures and Modular Decomposition of Graphs", Journal of
-Algorithms 16 (1994), pp. 283-294.
+Algorithms 16 (1994), pp. 283-294.  doi:10.1006/jagm.1994.1013
 
 I am not aware of any other publicly available implementations.
 Any errors and omissions are of course my fault.  Better algorithms
 are known: O(m+n) run-time can be achieved using sophisticated data
-structures (where m is the number of edges in the graph).  For a
-recent discussion of the history of modular decomposition, see
+structures (where m is the number of edges in the graph), see
 
-E. Dahlhaus, J. Gustedt and R. M. McConnell, "Partially Complemented
-Representations of Digraphs", Discrete Mathematics and Theoretical
-Computer Science 5 (2002), pp. 147-168.
+R. M. McConnell and F. de Montgolfier, "Linear-time modular
+decomposition of directed graphs", Discrete Applied Mathematics
+145 (2005), pp. 198-209.  doi:10.1016/j.dam.2004.02.017
 
 
 =head2 EXPORT
@@ -131,14 +130,14 @@ my $WSEP = '\|'; # regexp used to separate vertices
 my $PSEP = '\+'; # regexp used to separate elements of partition
 my $QSEP = '+'; # string used to separate elements of partition
 
-my $Debug = 0;
+my $MD_Debug = 0;
 
 sub debug {
     my $class = shift;
     if ( ref($class) ) { $class = ref($class) }
-    $Debug = shift;
-    carp 'Turning ', ($Debug ? 'on' : 'off'), ' ',
-	$class, ' debugging', ($Debug ? ", level $Debug" : '');
+    $MD_Debug = shift;
+    carp 'Turning ', ($MD_Debug ? 'on' : 'off'), ' ',
+	$class, ' debugging', ($MD_Debug ? ", level $MD_Debug" : '');
 }
 
 
@@ -146,7 +145,7 @@ sub debug {
 
     my $g = new Graph::ModularDecomposition;
     Graph::ModularDecomposition->canonical_form(1); # on (default)
-    Graph::ModularDecomposition->canonical_form(0); # turn it off
+    Graph::ModularDecomposition->canonical_form(0); # off
     $g->canonical_form(1); # same thing
     $g->canonical_form(0); # off
     print "yes" if $g->canonical_form();
@@ -161,7 +160,7 @@ no arguments, returns the current state.
 
 =cut
 
-my $Canonical_form = 0;
+my $Canonical_form = 1;
 
 sub canonical_form {
     my $class = shift;
@@ -178,7 +177,7 @@ sub canonical_form {
     $g = Graph::ModularDecomposition->new; # same thing
     my $h = $g->new;
 
-Constructor.  The instance method style C<$object->new> is an extension
+Constructor.  The instance method style C<$object>->C<new> is an extension
 and was not present in L<Graph::Directed|Graph::Directed>.
 
 =cut
@@ -186,7 +185,7 @@ and was not present in L<Graph::Directed|Graph::Directed>.
 sub new {
     my $self = shift;
     my $class = ref($self) ? ref($self) : $self;
-    return bless $class->SUPER::new(@_), $class;
+    return bless $class->SUPER::new(@_,directed=>1), $class;
 }
 
 
@@ -218,7 +217,7 @@ sub pairstring_to_graph {
     my $s = ( ( index( $pairs, '-' ) >= 0 ) ? '\-' : '' );
     foreach my $r ( split /,\s*/, $pairs ) {
 	( $p, $q ) = split $s, $r;
-	print "p=$p, q=$q\n" if $Debug > 2;
+	print "p=$p, q=$q\n" if $MD_Debug > 2;
 	if ( $q ) {
 	    $g = $g->add_edge( $p, $q ) unless $g->has_edge( $p, $q );
 	} else {
@@ -247,7 +246,7 @@ sub check_transitive {
     # get rid of loops
     foreach ( $h->vertices ) { $h->delete_edge( $_, $_ ) }
     foreach ( $g2->vertices ) { $g2->delete_edge( $_, $_ ) }
-    print STDERR "gdct: ", $g, ' vs. ', $h, "\n" if $Debug;
+    print STDERR "gdct: ", $g, ' vs. ', $h, "\n" if $MD_Debug;
     return $h eq $g2;
 }
 
@@ -265,11 +264,11 @@ sub setminus {
     my $X = shift;
     my $Y = shift;
     my @X = @{$X};
-    print STDERR 'setminus# ', @X, ' - ', @{$Y}, ' = ' if $Debug > 1;
+    print STDERR 'setminus# ', @X, ' - ', @{$Y}, ' = ' if $MD_Debug > 1;
     foreach my $x ( @{$Y} ) {
 	@X = grep $x ne $_, @X;
     }
-    print STDERR @X, "\n" if $Debug > 1;
+    print STDERR @X, "\n" if $MD_Debug > 1;
     return @X;
 }
 
@@ -288,11 +287,11 @@ sub setunion {
     my $X = shift;
     my $Y = shift;
     my @X = @{$X};
-    print STDERR 'setunion# ', @X, ' U ', @{$Y}, ' = ' if $Debug > 1;
+    print STDERR 'setunion# ', @X, ' U ', @{$Y}, ' = ' if $MD_Debug > 1;
     foreach my $x ( @{$Y} ) {
 	push @X, $x unless grep $x eq $_, @X;
     }
-    print STDERR @X, "\n" if $Debug > 1;
+    print STDERR @X, "\n" if $MD_Debug > 1;
     return sort @X;
 }
 
@@ -312,9 +311,9 @@ list of vertices.
 
 sub restriction {
     my $G = shift;
-    if ( $Debug > 2 ) { print STDERR 'restriction(', ref($G), ")\n" }
+    if ( $MD_Debug > 2 ) { print STDERR 'restriction(', ref($G), ")\n" }
     my $h = ($G->copy)->delete_vertices( setminus( [$G->vertices], [@_] ) );
-    if ( $Debug > 1 ) {
+    if ( $MD_Debug > 1 ) {
 	print STDERR 'restriction(', $G, '|', join($QSEP, @_), ') = ', $h, "\n"
     }
     return $h;
@@ -337,21 +336,21 @@ sub factor {
     my $GP = $G->copy;
     my $p;
     foreach my $X ( @{$P} ) {
-	print STDERR "factor# X = $X\n" if $Debug > 1;
-	print STDERR "factor# \@X = @$X\n" if $Debug > 1;
+	print STDERR "factor# X = $X\n" if $MD_Debug > 1;
+	print STDERR "factor# \@X = @$X\n" if $MD_Debug > 1;
 	my $newnode = join $VSEP, @{$X}; # turn nodes a, b, c into new node abc
-	print STDERR "factor# newnode = $newnode\n" if $Debug > 1;
+	print STDERR "factor# newnode = $newnode\n" if $MD_Debug > 1;
 	my $a = ${$X}[0];
-	print STDERR "factor# representative node $a\n" if $Debug > 1;
+	print STDERR "factor# representative node $a\n" if $MD_Debug > 1;
 	if ( $newnode ne $a ) { # do nothing if singleton
 	    $GP->add_vertex( $newnode );
 	    foreach $p ( $GP->predecessors( $a ) ) {
-		print STDERR "factor# predecessor $p\n" if $Debug > 2;
+		print STDERR "factor# predecessor $p\n" if $MD_Debug > 2;
 		$GP = $GP->add_edge( $p, $newnode )
 		    unless $GP->has_edge( $p, $newnode );
 	    }
 	    foreach $p ( $GP->successors( $a ) ) {
-		print STDERR "factor# successor $p\n" if $Debug > 2;
+		print STDERR "factor# successor $p\n" if $MD_Debug > 2;
 		$GP = $GP->add_edge( $newnode, $p )
 		    unless $GP->has_edge( $newnode, $p );
 	    }
@@ -376,25 +375,25 @@ sub partition_subsets {
     my $S = shift;
     my $w = shift;
 
-    print STDERR 'p..n_subsets# @S = ', @{$S}, ", w = $w \n" if $Debug > 1;
+    print STDERR 'p..n_subsets# @S = ', @{$S}, ", w = $w \n" if $MD_Debug > 1;
     my (@A, @B, @C, @D);
     foreach my $x ( @{$S} ) {
-	print STDERR 'p..n_subsets# xw = ', $x, $w if $Debug > 2;
+	print STDERR 'p..n_subsets# xw = ', $x, $w if $MD_Debug > 2;
 	if ( $G->has_edge( $w, $x ) ) {
 	    if ( $G->has_edge( $x, $w ) ) { # xw wx (not poset)
 		push @A, $x;
-		print STDERR ' A = ', @A, "\n" if $Debug > 2;
+		print STDERR ' A = ', @A, "\n" if $MD_Debug > 2;
 	    } else { # ~xw wx
 		push @B, $x;
-		print STDERR ' B = ', @B, "\n" if $Debug > 2;
+		print STDERR ' B = ', @B, "\n" if $MD_Debug > 2;
 	    }
 	} else {
 	    if ( $G->has_edge( $x, $w ) ) { # xw ~wx
 		push @C, $x;
-		print STDERR ' C = ', @C, "\n" if $Debug > 2;
+		print STDERR ' C = ', @C, "\n" if $MD_Debug > 2;
 	    } else { # ~xw ~wx
 		push @D, $x;
-		print STDERR ' D = ', @D, "\n" if $Debug > 2;
+		print STDERR ' D = ', @D, "\n" if $MD_Debug > 2;
 	    }
 	}
     }
@@ -415,26 +414,26 @@ sub partition {
     my $G = shift;
     my $v = shift;
 
-    print STDERR 'partition# G = ', $G, ", v = $v\n" if $Debug > 1;
+    print STDERR 'partition# G = ', $G, ", v = $v\n" if $MD_Debug > 1;
     my (%L, @done, $tempset, $S, @ZS, $w);
     $S = [ setminus( [ $G->vertices ], [ $v ] ) ];
-    print STDERR 'partition# @S = ', @{$S}, "\n" if $Debug > 1;
+    print STDERR 'partition# @S = ', @{$S}, "\n" if $MD_Debug > 1;
     $L{$S} = [ $v ];
     my @todo = ( $S );
-    print STDERR 'partition# L{S}[0] = ', $L{$S}[0], "\n" if $Debug > 1;
+    print STDERR 'partition# L{S}[0] = ', $L{$S}[0], "\n" if $MD_Debug > 1;
     while ( @todo ) {
 	$S = shift @todo;
 	@ZS = @{$L{$S}};
 	$w = $ZS[0];
-	print STDERR 'partition# ZS = ', @ZS, "\n" if $Debug > 1;
+	print STDERR 'partition# ZS = ', @ZS, "\n" if $MD_Debug > 1;
 	delete $L{$S};
 	foreach my $W ( $G->partition_subsets( $S, $w ) ) {
-	    print STDERR 'partition# W = ', @{$W}, "\n" if $Debug > 1;
+	    print STDERR 'partition# W = ', @{$W}, "\n" if $MD_Debug > 1;
 	    $tempset = [ setunion( [ setminus( $S, $W ) ],
 				[ setminus( \@ZS, [ $w ] ) ] ) ];
 	    if ( @{$tempset} ) {
 		print STDERR 'partition# tempset = ', @{$tempset}, "\n"
-		    if $Debug > 1;
+		    if $MD_Debug > 1;
 		$L{$W} = $tempset;
 		push @todo, $W;
 	    } else {
@@ -457,13 +456,13 @@ True if vertex $x distinguishes vertices $y and $z in graph $g.
 
 sub distinguishes {
     my ($g,$x,$y,$z) = @_;
-    print STDERR " $x$y?", $g->has_edge($x,$y) if $Debug > 1;
-    print STDERR " $x$z?", $g->has_edge($x,$z) if $Debug > 1;
-    print STDERR " $y$x?", $g->has_edge($y,$x) if $Debug > 1;
-    print STDERR " $z$x?", $g->has_edge($z,$x) if $Debug > 1;
+    print STDERR " $x$y?", $g->has_edge($x,$y) if $MD_Debug > 1;
+    print STDERR " $x$z?", $g->has_edge($x,$z) if $MD_Debug > 1;
+    print STDERR " $y$x?", $g->has_edge($y,$x) if $MD_Debug > 1;
+    print STDERR " $z$x?", $g->has_edge($z,$x) if $MD_Debug > 1;
     my $ret =  ( $g->has_edge($x,$y) != $g->has_edge($x,$z) )
 	    || ( $g->has_edge($y,$x) != $g->has_edge($z,$x) );
-    print STDERR "=$ret\n" if $Debug > 1;
+    print STDERR "=$ret\n" if $MD_Debug > 1;
     return $ret;
 }
 
@@ -482,20 +481,20 @@ sub G {
     my $g = shift;
     my $v = shift;
     my $G = new ref($g);
-    print STDERR 'G([', $g, "], $v) =...\n" if $Debug;
+    print STDERR 'G([', $g, "], $v) =...\n" if $MD_Debug;
 X:  foreach my $x ( $g->vertices ) {
 	next X if ( $v eq $x );
-	print STDERR 'X=', $x, "\n" if $Debug > 1;
+	print STDERR 'X=', $x, "\n" if $MD_Debug > 1;
 	$G = $G->add_vertex( $x );
 Y:	foreach my $y ( $g->vertices ) {
 	    next Y if ( $v eq $y or $x eq $y );
-	    print STDERR 'Y=', $y, "\n" if $Debug > 1;
+	    print STDERR 'Y=', $y, "\n" if $MD_Debug > 1;
 	    if ( $g->distinguishes( $x, $y, $v ) ) {
 		$G = $G->add_edge( $x, $y ) unless $G->has_edge( $x, $y );
 	    }
 	}
     }
-    print STDERR '...G()=', $G, "\n" if $Debug;
+    print STDERR '...G()=', $G, "\n" if $MD_Debug;
     return $G;
 }
 
@@ -576,13 +575,13 @@ sub modular_decomposition_EGMS {
     my $md = 0;
     $md ++;
     my $B = ' 'x$md;
-    print STDERR $B, 'MD(', $g, ")=...\n" if $Debug;
+    print STDERR $B, 'MD(', $g, ")=...\n" if $MD_Debug;
     my $v = ($g->vertices)[0];
-    print STDERR $B, 'v=', (defined($v) ? $v : 'undef'), "\n" if $Debug;
+    print STDERR $B, 'v=', (defined($v) ? $v : 'undef'), "\n" if $MD_Debug;
 
     my $t = {};
     unless ( $v ) {
-	print STDERR $B, '...MD=', tree_to_string( $t ), "\n" if $Debug;
+	print STDERR $B, '...MD=', tree_to_string( $t ), "\n" if $MD_Debug;
 	$md --;
 	return $t;
     }
@@ -596,7 +595,7 @@ sub modular_decomposition_EGMS {
     $t->{col} = '0';
 
     if ( scalar $g->vertices == 1 ) {
-	print STDERR $B, '...MD=', tree_to_string( $t ), "\n" if $Debug;
+	print STDERR $B, '...MD=', tree_to_string( $t ), "\n" if $MD_Debug;
 	$md --;
 	return $t;
     }
@@ -604,14 +603,14 @@ sub modular_decomposition_EGMS {
     my $p = partition( $g, $v );
     push @{$p}, [ $v ];
     my $gd = $g->factor( $p );
-    print STDERR $B, 'gd = ', $gd, "\n" if $Debug;
+    print STDERR $B, 'gd = ', $gd, "\n" if $MD_Debug;
     my $Gdd = $gd->G($v)->strongly_connected_graph;
-    print STDERR $B, 'Gdd = [', $Gdd, '], ', scalar $Gdd->vertices, "\n" if $Debug;
+    print STDERR $B, 'Gdd = [', $Gdd, '], ', scalar $Gdd->vertices, "\n" if $MD_Debug;
 
     my $u = $t;
     my @f;
     while ( @f = grep( $Gdd->out_degree($_) == 0 , $Gdd->vertices ) ) {
-	print STDERR $B, "\@f=[@f]\n" if $Debug;
+	print STDERR $B, "\@f=[@f]\n" if $MD_Debug;
 	my @s;
 	foreach my $s ( $Gdd->vertices ) {
 	    push @s, split(/$PSEP/, $s);
@@ -635,7 +634,7 @@ sub modular_decomposition_EGMS {
 		push @F, $F unless grep $F eq $_, @F;
 	    }
 	}
-	print STDERR $B, "\@F=@F\n" if $Debug;
+	print STDERR $B, "\@F=@F\n" if $MD_Debug;
 	if ( @f == 1 and @F > 1 ) {
 	    $u->{type} = 'primitive';
 	    $u->{col} = '0';
@@ -649,7 +648,7 @@ sub modular_decomposition_EGMS {
 		$u->{col} = '01';
 	    }
 	}
-	print STDERR $B, 'u = ', tree_to_string( $u ), "\n" if $Debug;
+	print STDERR $B, 'u = ', tree_to_string( $u ), "\n" if $MD_Debug;
 	foreach my $X ( @F ) {
 	    my $m = $g->restriction( split /$WSEP/, $X )
 		    ->modular_decomposition_EGMS;
@@ -660,7 +659,7 @@ sub modular_decomposition_EGMS {
 		   or ( $u->{type} eq 'linear' and $m->{type} eq 'linear' )
 		)
 	    ) {
-		if ( $Debug ) {
+		if ( $MD_Debug ) {
 		    print STDERR $B, "u->children= @{$u->{children}}\n";
 		    print STDERR $B, 'm->children= ';
 		    my $sep = '';
@@ -677,7 +676,7 @@ sub modular_decomposition_EGMS {
 	}
 	$u = $w;
     }
-    print STDERR $B, '...MD=', tree_to_string( $t ), "\n" if $Debug;
+    print STDERR $B, '...MD=', tree_to_string( $t ), "\n" if $MD_Debug;
     $md --;
     return $t;
 }
@@ -755,11 +754,11 @@ __END__
 
 =head1 AUTHOR
 
-Andras Salamon, E<lt>andras@dns.netE<gt>
+Andras Salamon, E<lt>azs@cpan.netE<gt>
 
 =head1 COPYRIGHT
 
-Copyright 2004-5, Andras Salamon.
+Copyright 2004-17, Andras Salamon.
 
 This code is distributed under the same copyright terms as Perl itself.
                                                                                 
